import axios, { AxiosInstance } from 'axios';
import auth from './authentication';
import { Environment } from './enum';

import { router } from 'expo-router';

export interface HttpClient extends AxiosInstance {
  url: string;
}

export const createHttpClient = (url: string, parent?: HttpClient) => {
  const baseUrl = parent ? parent.url + url : url;

  const httpClient = axios.create({
    baseURL: baseUrl,
    headers: {
      'content-type': 'application/json; charset=UTF-8',
      Accept: 'application/json',
    },
    paramsSerializer: { dots: true },
  }) as HttpClient;

  httpClient.url = baseUrl;

  httpClient.interceptors.request.use(async (config: any) => {
    const token = (await auth.getAutenticationToken()).token;
    if (config.headers && token) {
      config.headers.Authorization = `Bearer ${token}`;
    }
    return config;
  });

  httpClient.interceptors.response.use(
    (response) => response,
    (error) => {
      if (401 === error.response.status) {
        router.push({ pathname: '/home/', params: { signOff: 'true' } });
      } else {
        return Promise.reject(error);
      }
    }
  );

  return httpClient;
};

export const createStrapiHttpClient = (url: string, parent?: HttpClient) => {
  const baseUrl = parent ? parent.url + url : url;

  const httpClient = axios.create({
    baseURL: baseUrl,
    headers: {
      'content-type': 'application/json; charset=UTF-8',
      Accept: 'application/json',
    },
    paramsSerializer: { dots: true },
  }) as HttpClient;

  httpClient.url = baseUrl;

  httpClient.interceptors.request.use(async (config: any) => {
    switch (process.env.EXPO_PUBLIC_APP_ENV) {
      case Environment.Homologation:
        config.headers.Authorization = `Bearer 2f0ed95423faada59103d8fb46615c0c38a94a006edc762409d7e53b52fae7c19b777eaced1cb18e20fc853a6fdf37bc14a07475b1269f156567d9b12baf84481840c995aa84efb779800b57402b5b1854ac836ac5508cbfed84b0add93d6ff58cd137b5528fe3000f193d96376dd0b3da4875d89f0ca1f115787c7742809d0f`;
        break;
      case Environment.Development:
      case Environment.Local:
      default:
        config.headers.Authorization = `Bearer 4601324313abd1d6bae4a6f5a1727b9c09e8630d3bc17c4812606fc1ec4e7aafe9192f5d9531ce36693cc1fd4c5ad96300222f2d9a26fa0e85c6306cdbdbdc53fe3c398ee9de160d62ce064810ea84d78b3cd15f9feb61b684b62e49daa25a721c92f98b9b6a68f3a7a4c06ee19a453e5ef82a5753fb8174faf73382c71cf0c3`;
        break;
    }
    return config;
  });

  return httpClient;
};

