import { MaterialIcons } from '@expo/vector-icons';
import FontAwesome from '@expo/vector-icons/FontAwesome';
import AsyncStorage from '@react-native-async-storage/async-storage';
import { QueryClientProvider, useIsFetching, useIsMutating } from '@tanstack/react-query';
import { useFonts } from 'expo-font';
import { SplashScreen, Tabs } from 'expo-router';
import React, { useContext, useEffect, useMemo, useState } from 'react';
import { Platform } from 'react-native';
import { PaperProvider } from 'react-native-paper';
import { queryClient } from '../api/query-client';
import LoadingModal from '../components/loadingModal/loadingModal';
import { AuthContext, AuthProvider } from '../contexts/AuthContext';
import colors from '../styles/colors';
import theme from '../styles/theme';
import { WalkthroughContext, WalkthroughProvider } from '../contexts/WalkthroughContext';

export {
  // Catch any errors thrown by the Layout component.
  ErrorBoundary,
} from 'expo-router';

// Prevent the splash screen from auto-hiding before asset loading is complete.
SplashScreen.preventAutoHideAsync();

export default function RootLayout() {
  const [loaded, error] = useFonts({
    SpaceMono: require('../assets/fonts/RedHatText.ttf'),
    ...FontAwesome.font,
  });

  // Expo Router uses Error Boundaries to catch errors in the navigation tree.
  useEffect(() => {
    if (error) throw error;
  }, [error]);

  useEffect(() => {
    if (loaded) {
      SplashScreen.hideAsync();
    }
  }, [loaded]);

  if (!loaded) {
    return null;
  }

  return <Providers />;
}

function Providers() {
  return (
    <PaperProvider theme={theme}>
      <QueryClientProvider client={queryClient}>
        <AuthProvider>
          <WalkthroughProvider>
            <TabsLayout />
          </WalkthroughProvider>
        </AuthProvider>
      </QueryClientProvider>
    </PaperProvider>
  );
}

function TabsLayout() {
  const [authenticated, setAuthenticated] = useState(false);
  const isFetching = useIsFetching();
  const isMutating = useIsMutating();
  const authContext = useContext(AuthContext);
  const walkthroughContext = useContext(WalkthroughContext);

  const isLoading = useMemo(() => isFetching > 0 || isMutating > 0, [isFetching, isMutating]);

  useEffect(() => {
    AsyncStorage.getItem('TOKEN_API')
      .then((token) => {
        queryClient.invalidateQueries();
        setAuthenticated(token !== null);
      })
      .catch((error) => {
        console.error('Error reading TOKEN_API from AsyncStorage: ', error);
      });
  }, [authContext]);

  return (
    <>
      {isLoading && <LoadingModal />}
      <Tabs
        screenOptions={{
          headerShown: false,
          tabBarStyle: { height: Platform.OS === 'ios' ? 80 : 65 },
          tabBarLabelStyle: { paddingBottom: '5%' },
          tabBarHideOnKeyboard: true
        }}
        initialRouteName="home/index"
      >
        <Tabs.Screen
          name="home/index"
          options={{
            title: 'Início',
            tabBarActiveTintColor: colors.UT_ORANGE,
            tabBarInactiveTintColor: 'black',
            tabBarLabelPosition: 'below-icon',
            tabBarIcon: ({ size, focused }) => (
              <MaterialIcons id='home-tab-btn' ref={walkthroughContext.step2.ref} name="home" size={size} color={focused ? '#FB8500' : 'black'} />
            ),
          }}
        />
        <Tabs.Screen
          name="criminal_procedure_visualization/index"
          options={{
            title: 'Procedimentos Criminais',
            tabBarLabelPosition: 'below-icon',
            tabBarActiveTintColor: colors.UT_ORANGE,
            tabBarInactiveTintColor: 'black',
            href: authenticated ? '/criminal_procedure_visualization' : null,
            tabBarIcon: ({ size, focused }) => (
              <MaterialIcons
                id='criminal-procedures-tab-btn'
                name="insert-drive-file"
                ref={walkthroughContext.step3.ref}
                size={size}
                color={focused ? colors.UT_ORANGE : 'black'}
              />
            ),
          }}
        />
        <Tabs.Screen
          name="login/index"
          options={{
            title: authenticated ? 'Sair' : 'Entrar',
            tabBarActiveTintColor: colors.UT_ORANGE,
            tabBarInactiveTintColor: 'black',
            tabBarLabelPosition: 'below-icon',
            tabBarIcon: ({ size, focused }) => (
              <MaterialIcons
								ref={walkthroughContext.step4.ref}
								id='login-tab-btn'
                name={authenticated ? 'logout' : 'login'}
                size={size}
                color={focused ? colors.UT_ORANGE : 'black'}
              />
            ),
            href: {
              pathname: authenticated ? '/home' : '/login',
              params: authenticated
                ? {
                  signOff: true,
                }
                : {},
            },
          }}
        />
        <Tabs.Screen name="register_verification/FailScreen" options={{ href: null }} />
        <Tabs.Screen name="register_verification/SuccessScreen" options={{ href: null }} />
        <Tabs.Screen name="signup/index" options={{ href: null }} />
        <Tabs.Screen name="criminal_procedure_register/index" options={{ href: null }} />
        <Tabs.Screen name="procedure_number/index" options={{ href: null }} />
        <Tabs.Screen name="index" options={{ href: null }} />
        <Tabs.Screen name="documents/index" options={{ href: null }} />
        <Tabs.Screen name="upload_files/index" options={{ href: null }} />
      </Tabs>
    </>
  );
}

