import { router } from 'expo-router';
import React, { useMemo, useState } from 'react';
import { Modal, ScrollView, StyleSheet, View } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { Text } from 'react-native-paper';
import { useGetCities } from '../../api/api/cities/queries';
import { useGetCrimes } from '../../api/api/crimes/queries';
import { useCreateCriminalProcedure } from '../../api/api/criminal-procedures/requests';
import ExclamationCircle from '../../assets/images/exclamation-circle.svg';
import ButtonComponent from '../../components/button/component_button';
import DatePicker from '../../components/datepicker/datepicker';
import Dropdown, { Option } from '../../components/dropdrown/dropdown';
import Exception from '../../components/error/error';
import CustomHeader from '../../components/header/header';
import InputText from '../../components/input/input';
import RadioButtonComponent from '../../components/radioButton/radioButtonComponent';
import colors from '../../styles/colors';

interface Form {
  procedureNumber: string;
  date: string;
  description: string;
  city: Option;
  crime: Option;
  procedureType: string;
  customCrimeType: string;
}

export const PROCEDURE_TYPE_OPTIONS = [
  {
    key: 'OCCURRENCE',
    value: 'Ocorrência',
  },
  {
    key: 'PROCESS',
    value: 'Processo',
  },
];

const DEFAULT_FORM = {
  procedureNumber: '',
  date: new Date().toLocaleString().slice(0, 10).replaceAll(',', ''),
  description: '',
  city: {} as Option,
  crime: {} as Option,
  procedureType: '',
  customCrimeType: '',
};

export default function FormComponent() {
  const { data: citiesData } = useGetCities(true);
  const { data: crimesData } = useGetCrimes(true);

  const createProcedureMutation = useCreateCriminalProcedure();

  const [popupVisible, setPopupVisible] = useState(true);

  const [form, setForm] = useState<Form>(DEFAULT_FORM);

  const [formErrors, setFormErrors] = useState({
    procedureNumber: '',
    date: '',
    description: '',
    city: '',
    crime: '',
    procedureType: '',
    customCrimeType: '',
  });

  const [error, setError] = useState(false);

  const isCrimeOther = useMemo(() => form.crime.name === 'Outro', [form.crime]);

  const validateProcedureType = (procedureType: string) => {
    return procedureType === '' ? 'Por favor, selecione uma opção válida' : '';
  }

  const validateCrime = (crime: { name: string; }, customCrimeType: string) => {
    if (!crime) return 'Por favor, selecione uma opção válida';
    if (crime.name === 'Outro' && customCrimeType === '') return 'Por favor, detalhe o tipo de crime';
    return '';
  }

  const validateDate = (date: string) => {
    return date === '' ? 'Por favor, selecione uma data válida' : '';
  };

  const validateProcedureNumber = (procedureNumber: string | any[]) => {
    return procedureNumber === '' || procedureNumber.length !== 20 ?
      'Por favor, entre um número de procedimento válido' : '';
  };

  const validateCity = (city: any) => {
    return !city ? 'Por favor, selecione uma cidade válida' : '';
  };

  const validateDescription = (description: string) => {
    return description === '' ? 'Por favor, entre uma descrição válida' : '';
  };

  const validateForm = () => {
    const errors = {
      procedureType: validateProcedureType(form.procedureType),
      crime: validateCrime(form.crime, form.customCrimeType),
      date: validateDate(form.date),
      procedureNumber: validateProcedureNumber(form.procedureNumber),
      city: validateCity(form.city),
      description: validateDescription(form.description),
      customCrimeType: form.crime.name === 'Outro' && form.customCrimeType === '' ?
        'Por favor, detalhe o tipo de crime' : ''
    };
    setFormErrors(errors);
    return errors;
  }

  const handleProcedureType = (e: string) => {
    setForm((prevState: Form) => ({
      ...prevState,
      procedureType: e,
    }));
  };

  const handleDateChange = (e: string) => {
    setForm((prevState: Form) => ({
      ...prevState,
      date: e,
    }));
  };

  const handleCrimeTypeChange = (e: Option) => {
    setForm((prevState: Form) => ({
      ...prevState,
      crime: e,
    }));
    if (e.name !== 'Outro') {
      setForm((prevState: Form) => ({
        ...prevState,
        customCrimeType: '',
      }));
    }
  };

  const customCrimeChange = (e: string) => {
    setForm((prevState: Form) => ({
      ...prevState,
      customCrimeType: e,
    }));
  };

  const handleCityChange = (e: Option) => {
    setForm((prevState: Form) => ({
      ...prevState,
      city: e,
    }));
  };

  const handleProcedureNumberChange = (e: string) => {
    setForm((prevState: Form) => ({
      ...prevState,
      procedureNumber: e,
    }));
  };

  const handleSetDescription = (e: string) => {
    setForm((prevState: Form) => ({
      ...prevState,
      description: e,
    }));
  };

  const handleSubmit = () => {
    const errors = validateForm();

    if (Object.values(errors).some((err) => err !== '')) return;

    const { city, crime, ...createForm } = {
      ...form,
      date: form.date.split(', ')[0].split('/').reverse().join('-').replace(/\b\d\b/g, match => match.padStart(2, '0')),
      procedureType: form.procedureType === 'Ocorrência' ? 'OCCURRENCE' : 'PROCESS',
      cityId: form.city.id,
      crimeId: form.crime.id,
    };

    createProcedureMutation.mutate(createForm, {
      onSuccess: () => {
        setForm(DEFAULT_FORM);
        router.replace('/criminal_procedure_visualization/');
      },
      onError: () => {
        setError(true);
      },
    });
  };

  const togglePopup = () => {
    setPopupVisible(!popupVisible);
  };

  if (error) {
    return <Exception handleTryAgain={() => setError(false)} />;
  }

  return (
    <ScrollView style={{height: '100%'}}>
      <Modal transparent visible={popupVisible}>
        <View style={styles.modal}>
          <View style={styles.modalAtention}>
            <ExclamationCircle />
            <Text style={{ color: colors.UT_ORANGE, fontWeight: 'bold', height: 20 }}>ATENÇÃO</Text>
            <Text style={{ textAlign: 'center', height: 50 }}>
              Enviar denúncias e relatos falsos é considerado crime passível de pena/multa, de
              acordo com o Artigo 340 do Código Penal. Por favor, use o aplicativo com
              responsabilidade.
            </Text>
            <ButtonComponent value={'Fechar'} onClick={togglePopup} id='btn-close-crim-proc-warning'></ButtonComponent>
          </View>
        </View>
      </Modal>


        <View style={styles.header}>
          <CustomHeader
            title="Procedimento Criminal"
            description="Preencha os dados abaixo para adicionar um novo procedimento criminal."
          />
        </View>
        <KeyboardAwareScrollView
        automaticallyAdjustKeyboardInsets
        style={{ zIndex: -1, flex: 1, height: '100%' }}
        contentContainerStyle={{ flex: 1 }}
      >
        <View style={styles.container}>
          <View style={styles.radio}>
            {PROCEDURE_TYPE_OPTIONS.map((option) => (
              <RadioButtonComponent
								id={option.key + "-option"}
                key={option.key}
                value={form.procedureType === option.value}
                label={option.value}
                onValueChange={() => handleProcedureType(option.value)}
              />
            ))}
          </View>
          {formErrors.procedureType !== '' && (
            <Text style={styles.inputErrorText}>{formErrors.procedureType}</Text>
          )}

          <Dropdown
						id='crime-type-form-input'
            placeholder="Você foi vítima de qual crime?"
            selectedValue={form.crime.name}
            options={crimesData}
            onSelect={handleCrimeTypeChange}
          />
          {formErrors.crime !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.crime}
            </Text>
          )}

          {isCrimeOther && (
            <InputText
              label="Explique o ocorrido"
              state={form.customCrimeType}
              handleChange={customCrimeChange}
              isInvalid={formErrors.customCrimeType !== ''}
              maxLength={30}
              withTooltip
              tooltipText="Caso não saiba o tipo do crime, explique em até 30 caracteres o que aconteceu. Você poderá dar mais detalhes no campo de descrição abaixo." // Adicionando a tooltip
            />
          )}

          {formErrors.customCrimeType !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.customCrimeType}
            </Text>
          )}

          <DatePicker onSelectDate={handleDateChange} value={form.date}  />
          {formErrors.date !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.date}
            </Text>
          )}

          <InputText
            id='criminal-procedure-number-form-input'
            label="Informe o n° do proc. criminal"
            state={form.procedureNumber}
            handleChange={handleProcedureNumberChange}
            isInvalid={false}
            withTooltip
            tooltipText="Para adicionar um procedimento criminal, é necessário ter um Boletim de Ocorrência registrado ou um processo em andamento. 
          Para mais informações, clique aqui."
          />
          {formErrors.procedureNumber !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.procedureNumber}
            </Text>
          )}

          <Dropdown
						id='criminal-procedure-location-form-input'
            placeholder="Onde ocorreu o crime?"
            selectedValue={form.city.name}
            options={citiesData}
            onSelect={handleCityChange}
          />
          {formErrors.city !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.city}
            </Text>
          )}

          <InputText
						id='criminal-procedure-details-form-input'
            label="Dê mais detalhes ao ocorrido"
            state={form.description}
            handleChange={handleSetDescription}
            isInvalid={false}
            size={2}
          />

          {formErrors.description !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.description}
            </Text>
          )}
          <View style={{ width: '80%', paddingBottom: 40 }}>
            <ButtonComponent id='criminal-procedure-submit-form-btn' value={'Cadastrar'} onClick={handleSubmit} />
          </View>
        </View>
      </KeyboardAwareScrollView>  
    </ScrollView>
  );
}

const styles = StyleSheet.create({
  container: {
    display: 'flex',
    alignItems: 'center',
    paddingHorizontal: 23,
    flex: 1,
    gap: 40
  },

  header: {
    zIndex: -1,
  },

  radio: {
    display: 'flex',
    flexDirection: 'row',
    justifyContent: 'space-between',
    gap: 30,
    width: '100%',
  },
  icon: {
    color: colors.SELECTIVE_YELLOW,
  },
  modalAtention: {
    width: '90%',
    borderRadius: 10,
    padding: '7%',
    gap: 20,
    elevation: 20,
    justifyContent: 'center',
    alignItems: 'center',
    backgroundColor: colors.BACKGROUND,
    zIndex: 2,
  },

  modal: {
    backgroundColor: 'rgba(0, 0, 0, 0.5)',
    alignItems: 'center',
    justifyContent: 'center',
    flex: 1,
  },

  inputErrorText: {
    color: colors.VERMILION,
    marginTop: -5,
    fontSize: 12,
    marginLeft: '2%',
  },
});

