import React, { useContext, useEffect, useRef, useState } from 'react';
import Accordion from '../../components/accordion/Accordion';
import colors from '../../styles/colors';
import { Image, SafeAreaView, ScrollView, StyleSheet, TouchableOpacity, View } from 'react-native';
import { Text } from 'react-native-paper';
import general from '../../styles/general';
import UserQuestion from '../../assets/images/faq.svg';
import InfoCard from '../../components/infoCards/infoCards';
import { IconButton } from 'react-native-paper';
import { useLocalSearchParams } from 'expo-router';
import { AuthContext } from '../../contexts/AuthContext';
import { useGetKnowYourRights } from '../../api/strapi/kyr/queries';
import { useGetFrequentlyAskedQuestions } from '../../api/strapi/faq/queries';
import { getEnvironmentRoute } from '../../api';
import { UrlType } from '../../api/enum';
import { MaterialIcons } from '@expo/vector-icons';
import WhatsappButton from '../../components/whatsappButton/whatsappButton';
import Walkthrough from '../../components/walkthrough/walkthrough';
import { WalkthroughContext } from '../../contexts/WalkthroughContext';

const FaqPage = () => {
  const params = useLocalSearchParams();

  const { signOut } = useContext(AuthContext);
  const { data: kyr } = useGetKnowYourRights(true);
  const { data: faq } = useGetFrequentlyAskedQuestions(true);

  const [isWalkthroughOpen, setIsWalkthroughOpen] = useState(false);
  const walkthroughContext = useContext(WalkthroughContext);

  useEffect(() => {
    if (params.signOff == 'true') {
      signOut();
    }
  }, [params]);

  return (
    <SafeAreaView
      style={{
        height: '100%',
        width: '100%',
        backgroundColor: colors.BACKGROUND,
      }}
    >
      <ScrollView style={styles.container}>
        <Walkthrough
          walkthroughOpen={isWalkthroughOpen}
          setIsWalkthroughOpen={setIsWalkthroughOpen}
        />
        <View style={styles.headerHelp}>
          <TouchableOpacity
						id='walkthrough-start-btn'
            onPress={() => {
              setIsWalkthroughOpen(true);
            }}
          >
            <MaterialIcons
              ref={walkthroughContext.step1.ref}
              name="help-outline"
              color={'#FB8500'}
              size={24}
            />
          </TouchableOpacity>

          <WhatsappButton />
        </View>
        <IconButton style={styles.faqImage} icon={UserQuestion} size={60} />
        <Text id='rights-section-header-text' style={[styles.faqText, { fontWeight: 'bold' }]}>Conheça seus direitos</Text>
        {kyr && kyr.data && (
          <ScrollView
            style={styles.carrossel}
            horizontal={true}
            showsHorizontalScrollIndicator={true}
          >
            {kyr.data.map(
              (item: {
                id: string;
                attributes: {
                  title: string;
                  content: string;
                  logo: {
                    data: {
                      attributes: {
                        width: number;
                        height: number;
                        url: string;
                      };
                    };
                  };
                };
              }) => (
                <React.Fragment key={item.id}>
                  <View style={styles.infoCardContainer}>
                    <InfoCard
                      title={item.attributes.title}
                      description={item.attributes.content}
											id={'kyr-card-' + item.id}
                      Icon={() => (
                        <Image
                          style={{
                            width: item.attributes.logo.data.attributes.width,
                            height: item.attributes.logo.data.attributes.height,
                          }}
                          source={{
                            uri:
                              getEnvironmentRoute(UrlType.STRAPI) +
                              item.attributes.logo.data.attributes.url,
                          }}
                        />
                      )}
                    />
                  </View>
                </React.Fragment>
              )
            )}
          </ScrollView>
        )}

        <View style={styles.iconTextContainer}>
          <MaterialIcons name="help-outline" color={'#FB8500'} size={28} />
          <Text id='faq-section-header' style={styles.faqText}>Perguntas Frequentes</Text>
        </View>

        <View style={styles.faqContainer}>
          {faq &&
            faq.data &&
            faq.data.map((data: { id: string; attributes: { title: string; content: string } }) => (
              <React.Fragment key={data.id}>
                <Accordion id={'faq-accordion-' + data.id} title={data.attributes.title} description={data.attributes.content} />
              </React.Fragment>
            ))}
        </View>
      </ScrollView>
    </SafeAreaView>
  );
};

const styles = StyleSheet.create({
  items_container: {
    overflowY: 'hidden',
    width: '100%',
    marginHorizontal: 'auto',
    height: '100%',
    padding: 10,
  },
  headerHelp: {
		alignSelf: 'center',
    width: '60%',
    marginTop: '8%',
    display: 'flex',
    flexDirection: 'row',
    justifyContent: 'space-between',
    transform: 'scale(1.4)',
  },
  row: {
    flexDirection: 'row',
    alignItems: 'center',
  },
  scrollContainer: {
    alignItems: 'center',
    color: colors.SELECTIVE_YELLOW,
  },
  smallText: {
    color: colors.EERIE_BLACK,
    marginVertical: 5,
    fontSize: general.title4.fontSize,
    fontWeight: general.title4.fontWeight,
  },
  largeText: {
    color: colors.EERIE_BLACK,
    textAlign: 'center',
    padding: 10,
    fontSize: general.body3.fontSize,
    fontWeight: general.body3.fontWeight,
    lineHeight: 25,
  },
  cards: {
    display: 'flex',
    justifyContent: 'center',
    flexDirection: 'row',
    alignContent: 'center',
    alignItems: 'center',
    marginVertical: 15,
  },
  carrossel: {
    flexDirection: 'row',
    paddingHorizontal: 10,
  },
  infoCardContainer: {
    marginRight: 10,
  },
  faqText: {
    color: colors.EERIE_BLACK,
    fontSize: general.title1.fontSize,
    fontWeight: general.title1.fontWeight,
    alignItems: 'center',
    textAlign: 'center',
    lineHeight: 32,
    marginLeft: 2,
  },
  faqImage: {
    alignSelf: 'center',
    width: 120,
    height: 120,
  },
  iconTextContainer: {
    display: 'flex',
    flexDirection: 'row',
    alignItems: 'flex-end',
    marginLeft: 10,
    marginTop: 30,
  },
  container: {
    width: '100%',
    marginHorizontal: 'auto',
    height: '100%',
    padding: 10,
  },
  searchContainer: {
    marginTop: 10,
    display: 'flex',
    alignSelf: 'center',
    flexDirection: 'column',
    alignItems: 'flex-start',
    justifyContent: 'center',
    width: '90%',
  },
  walkthroughContainer: {
    marginTop: 40,
    display: 'flex',
    alignSelf: 'center',
    flexDirection: 'column',
    alignItems: 'flex-end',
    justifyContent: 'center',
    width: '90%',
  },
  faqContainer: {
    display: 'flex',
    flexDirection: 'column',
    justifyContent: 'center',
    alignItems: 'center',
    marginVertical: 10,
    gap: 20,
  },
});

export default FaqPage;
