import { Link, router } from 'expo-router';
import { default as React, useContext, useState } from 'react';
import { ScrollView, StyleSheet, Text, View } from 'react-native';
import { Credentials, useLogin } from '../../api/auth/login/requests';
import Identification from '../../assets/images/identification.svg';
import LockClosed from '../../assets/images/lock-closed.svg';
import UserCircle from '../../assets/images/user-cicle.svg';
import Button from '../../components/button/component_button';
import InputText from '../../components/input/input';
import { AuthContext } from '../../contexts/AuthContext';
import colors from '../../styles/colors';
import general from '../../styles/general';

export let setTimeLoader = 4000;

export default function Render() {
  const [credentials, setCredentials] = useState<Credentials>({
    cpf: '',
    password: '',
  });

  const loginMutation = useLogin();

  const { signIn } = useContext(AuthContext);

  const [isCredentialsValid, setCredentialsValid] = useState(true);

  const onCPFChange = (e: string) => {
    setCredentialsValid(true);

    const numericValue = e.replace(/\D/g, '');
    if (numericValue.length > 11) return;

    setCredentials({ ...credentials, cpf: numericValue });

    if (numericValue.length == 11) {
      const formattedNumber = `${numericValue.slice(0, 3)}.${numericValue.slice(
        3,
        6
      )}.${numericValue.slice(6, 9)}-${numericValue.slice(9, 11)}`;
      setCredentials({ ...credentials, cpf: formattedNumber });
    }
  };

  const onPasswordChange = (e: any) => {
    setCredentialsValid(true);
    setCredentials((prevState) => ({
      ...prevState,
      password: e,
    }));
  };

  const handleLoginClick = () => {
    (async function () {
      loginMutation.mutate(credentials, {
        onSuccess: ({ data }) => {
          signIn(data.access_token);
          router.replace('/home/');
        },
        onError: (error) => {
          setCredentialsValid(false);
        },
      });
    })();
  };

  return (
    <ScrollView style={styles.mainContainer}>
      <View style={styles.iconContainer}>
        <UserCircle />
      </View>

      <Text style={styles.title}>Entrar</Text>
      <View style={styles.textContainer}>
        <Text style={styles.description}>
          Por favor, insira os seus dados abaixo para continuar.
        </Text>
      </View>

      <View style={styles.inputContainer}>
        <InputText
          label="CPF"
          state={credentials.cpf}
          handleChange={onCPFChange}
          leftIcon={<Identification />}
          isInvalid={!isCredentialsValid}
					id="login-cpf-field"
        />

        <InputText
          handleChange={onPasswordChange}
          state={credentials.password}
          label="Senha"
          leftIcon={<LockClosed />}
          type="password"
          isInvalid={!isCredentialsValid}
					id="login-password-field"
        />
      </View>

      <View style={styles.loginContainer}>
        <Text
          style={{
            display: isCredentialsValid ? 'none' : 'flex',
            ...styles.textError,
          }}
        >
          Credenciais incorretas
        </Text>
        <View style={styles.buttonContainer}>
          <Button id='login-button' value={'Entrar'} onClick={handleLoginClick} />
        </View>
      </View>

      <View style={styles.noAccountContainer}>
        <Text style={{ ...styles.mediumText }}>
          Não possui uma conta?{' '}
          <Link href="/signup/" style={{ color: colors.UT_ORANGE }} id='signUp-hiperlink'>
            Cadastrar
          </Link>
        </Text>
      </View>
    </ScrollView>
  );
}

const styles = StyleSheet.create({
  mainContainer: {
    backgroundColor: colors.BACKGROUND,
    paddingTop: 40,
    paddingHorizontal: 30,
    paddingBottom: 30,
    height: '100%',
  },
  mediumText: {
    textAlign: 'center',
    fontSize: general.body2.fontSize,
    fontWeight: general.body2.fontWeight,
    color: colors.EERIE_BLACK,
  },
  iconContainer: {
    alignItems: 'center',
    height: 150,
    justifyContent: 'center',
    marginBottom: '5%',
  },
  textContainer: {
    alignItems: 'center',
    marginBottom: '10%',
  },
  title: {
    fontWeight: general.header2.fontWeight,
    fontSize: general.header2.fontSize,
    marginBottom: 5,
    alignSelf: 'center',
  },
  description: {
    textAlign: 'center',
    fontSize: general.body2.fontSize,
    fontWeight: general.body2.fontWeight,
  },
  inputContainer: {
    display: 'flex',
    alignSelf: 'center',
    alignContent: 'center',
    alignItems: 'center',
    width: '100%',
    gap: 20,
  },
  loginContainer: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    marginTop: '4.5%',
  },
  buttonContainer: {
    width: '80%',
  },
  forgetPasswordContainer: {
    alignSelf: 'flex-end',
  },
  noAccountContainer: {
    justifyContent: 'center',
    flexDirection: 'row',
    marginTop: '4%',
  },
  textLink: { color: colors.UT_ORANGE },

  textError: {
    color: colors.VERMILION,
    marginBottom: 10,
  },
});

