import { Link, router } from 'expo-router';
import React, { useState } from 'react';
import { StyleSheet, View } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { Text } from 'react-native-paper';
import Email from '../../assets/images/email.svg';
import Identification from '../../assets/images/identification.svg';
import LockClosed from '../../assets/images/lock-closed.svg';
import Phone from '../../assets/images/phone.svg';
import UserCircle from '../../assets/images/user-cicle.svg';
import User from '../../assets/images/user.svg';
import ComponentButton from '../../components/button/component_button';
import Hint from '../../components/input/hint';
import InputText from '../../components/input/input';
import colors from '../../styles/colors';
import general from '../../styles/general';
import { isCPFValid } from '../../validators/CPFValidator';

import {
  checkPasswordCase,
  checkPasswordLength,
  checkPasswordNumber,
  checkPasswordsEquals,
} from '../../validators/PasswordValidator';

const Signup = () => {
  const [form, setForm] = useState({
    name: '',
    cpf: '',
    email: '',
    phone: '',
    password: '',
    repeatPassword: '',
  });
  const [formErrors, setFormErrors] = useState({
    name: '',
    cpf: '',
    email: '',
    phone: '',
    password: '',
    repeatPassword: '',
  });

  const handleSignup = () => {
    validateForm();

    if (Object.values(formErrors).some((err) => err !== '')) return;

    router.push({ pathname: '/procedure_number/', params: form });
  };

  const handleNameChange = (name: string) => {
    setForm({ ...form, name: name });
  };

	const validateForm = () => {
		let errors = { ...formErrors };
	
		errors.name = validateName(form.name);
		errors.phone = validatePhone(form.phone);
		errors.cpf = validateCPF(form.cpf);
		errors.email = validateEmail(form.email);
		errors.password = validatePassword(form.password);
		errors.repeatPassword = validateRepeatPassword(form.password, form.repeatPassword);
	
		setFormErrors(errors);
	};
	
	const validateName = (name: string) => {
		const nameRegex = /\s/;
		if (name.length === 0 || !nameRegex.test(name)) {
			return 'Por favor, informe um nome válido';
		}
		return '';
	};
	
	const validatePhone = (phone: string) => {
		if (phone.replace(/\D/g, '').length !== 11) {
			return 'Por favor, informe um telefone válido';
		}
		return '';
	};
	
	const validateCPF = (cpf: string) => {
		if (!isCPFValid(cpf.replace(/\D/g, ''))) {
			return 'Por favor, informe um CPF válido';
		}
		return '';
	};
	
	const validateEmail = (email: string) => {
		const emailRegex = /\S+@\S+\.\S+/;
		if (email.length === 0 || !emailRegex.test(email)) {
			return 'Por favor, informe um e-mail válido';
		}
		return '';
	};
	
	const validatePassword = (password: string) => {
		if (!checkPasswordLength(password)) {
			return 'A senha deve conter no mínimo 8 caracteres.';
		}
		if (!checkPasswordCase(password)) {
			return 'A senha deve conter letras maiúsculas e minúsculas.';
		}
		if (!checkPasswordNumber(password)) {
			return 'A senha deve conter números.';
		}
		if (password.length === 0) {
			return 'Por favor, informe uma senha válida';
		}
		return '';
	};
	
	const validateRepeatPassword = (password: string, repeatPassword: string) => {
		if (repeatPassword.length === 0) {
			return 'Por favor, confirme sua senha';
		}
		if (password !== repeatPassword) {
			return 'As senhas devem coincidir.';
		}
		return '';
	};
	

  const handlePhoneChange = (input: string) => {
    const numericValue = input.replace(/\D/g, '');
    setForm({ ...form, phone: numericValue });

    if (numericValue.length == 11) {
      const formattedNumber = `(${numericValue.slice(0, 2)}) ${numericValue.slice(
        2,
        7
      )}-${numericValue.slice(7, 11)}`;
      setForm({ ...form, phone: formattedNumber });
    }
  };

  const formatCPFNumber = (input: string) => {
    const numericValue = input.replace(/\D/g, '');
    setForm({ ...form, cpf: numericValue });

    if (numericValue.length === 11) {
      const formattedNumber = `${numericValue.slice(0, 3)}.${numericValue.slice(
        3,
        6
      )}.${numericValue.slice(6, 9)}-${numericValue.slice(9, 11)}`;
      setForm({ ...form, cpf: formattedNumber });
    }
  };

  const handleEmailChange = (email: string) => {
    setForm({ ...form, email: email });
  };

  const handlePasswordChange = (password: string) => {
    setForm({ ...form, password: password });
  };

  const isPasswordValid = () => {
    return (
      checkPasswordLength(form.password) &&
      checkPasswordCase(form.password) &&
      checkPasswordNumber(form.password)
    );
  };

  const styles = StyleSheet.create({
    container: {
      backgroundColor: colors.BACKGROUND,
      height: '100%',
      width: 'auto',
    },
    topContainer: {
      paddingTop: '10%',
      display: 'flex',
      justifyContent: 'center',
      alignItems: 'center',
      gap: 20,
    },
    inputsContainer: {
      marginTop: '3.5%',
      alignSelf: 'center',
      alignContent: 'center',
      width: '80%',
      justifyContent: 'space-between',
      flex: 1,
      gap: 10,
    },
    signUpContainer: {
      marginTop: '3.5%',
      marginBottom: 10,
      alignSelf: 'center',
      paddingBottom: '3.5%',
      width: '90%',
      paddingHorizontal: 23,
    },
    inputErrorText: {
      color: colors.VERMILION,
      marginTop: -5,
      fontSize: 12,
      marginLeft: '2%',
    },
    largeText: {
      textAlign: 'center',
      fontSize: general.header2.fontSize,
      fontWeight: general.header2.fontWeight,
      color: colors.EERIE_BLACK,
    },
    mediumText: {
      textAlign: 'center',
      paddingHorizontal: 23,
      fontSize: 16,
      color: colors.EERIE_BLACK,
    },
    InputTextContainer: {
      width: '75%',
      textAlign: 'left',
      display: 'flex',
      flexDirection: 'column',
      alignItems: 'flex-start',
    },
    passwordHintWrapper: {
      display: 'flex',
      flexDirection: 'row',
      alignItems: 'center',
      gap: 40,
    },
    mediumErrorText: {
      marginLeft: '3%',
      marginTop: -5,
      fontSize: 14,
      color: '#6C6C6C',
    },
  });

  return (
    <KeyboardAwareScrollView style={{ flex: 1 }}>
      <View style={styles.container}>
        <View style={styles.topContainer} id='signUp-header'>
          <UserCircle style={{ width: '150px', height: '150px' } as any} />
          <Text variant="titleLarge" style={styles.largeText}>
            Cadastrar novo usuário
          </Text>

          <Text variant="bodyMedium" style={styles.mediumText}>
            Por favor, insira os seus dados abaixo para continuar.
          </Text>
        </View>

        <View style={styles.inputsContainer}>
          <InputText
            state={form.name}
            handleChange={(val) => handleNameChange(val)}
            label="Nome"
            leftIcon={<User />}
            isInvalid={formErrors.name !== ''}
						id="name-field"
          />
          {formErrors.name !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.name}
            </Text>
          )}

          <InputText
            state={form.cpf}
            handleChange={(val) => formatCPFNumber(val)}
            label="CPF"
            leftIcon={<Identification />}
            isInvalid={formErrors.cpf !== ''}
						id="signup-cpf-field"
          />
          {formErrors.cpf !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.cpf}
            </Text>
          )}

          <InputText
            state={form.email}
            handleChange={(val) => handleEmailChange(val)}
            label="E-mail"
            leftIcon={<Email />}
            isInvalid={formErrors.email !== ''}
						id="email-field"
          />
          {formErrors.email !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.email}
            </Text>
          )}
          <InputText
            state={form.phone}
            handleChange={(state) => handlePhoneChange(state)}
            label="Telefone"
            leftIcon={<Phone />}
            isInvalid={formErrors.phone !== ''}
						id="phone-field"
          />
          {formErrors.phone !== '' && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.phone}
            </Text>
          )}

          <InputText
            state={form.password}
            handleChange={(val) => handlePasswordChange(val)}
            label="Senha"
            leftIcon={<LockClosed />}
            type="password"
            isInvalid={form.password !== '' && !isPasswordValid()}
						id="password-field"
          />
          {form.password.length === 0 && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.password}
            </Text>
          )}
          {form.password !== '' && !isPasswordValid() && (
            <View style={styles.InputTextContainer}>
              <Text variant="bodyMedium" style={styles.mediumErrorText}>
                Sua senha deve conter:
              </Text>
              <Hint
                type={1}
                label="No mínimo 8 caracteres"
                isValid={checkPasswordLength(form.password)}
              />
              <Hint
                type={1}
                label="Letras maiúsculas e minúsculas"
                isValid={checkPasswordCase(form.password)}
              />
              <Hint type={1} label="Números" isValid={checkPasswordNumber(form.password)} />
            </View>
          )}

          <InputText
            state={form.repeatPassword}
            handleChange={(val) => setForm({ ...form, repeatPassword: val })}
            label="Confirme sua senha"
            leftIcon={<LockClosed />}
            type="password"
            isInvalid={!checkPasswordsEquals(form.password, form.repeatPassword)}
						id="password-check-field"
          />
          {form.repeatPassword.length === 0 && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              {formErrors.repeatPassword}
            </Text>
          )}
          {!checkPasswordsEquals(form.password, form.repeatPassword) && (
            <Text variant="bodyMedium" style={styles.inputErrorText}>
              As senhas não coincidem
            </Text>
          )}

          <View style={styles.signUpContainer}>
            <ComponentButton onClick={() => handleSignup()} value="Cadastrar" id='signUp-button'/>

            <Text
              variant="bodyMedium"
              style={{ ...styles.mediumText, display: 'flex', width: '100%', marginTop: 15 }}
            >
              Já possui uma conta?{' '}
              <Link href="/login/" style={{ color: colors.UT_ORANGE }}>
                Entrar
              </Link>
            </Text>
          </View>
        </View>
      </View>
    </KeyboardAwareScrollView>
  );
};
export default Signup;

