import React, { useRef, useState } from 'react';
import EyeOff from '../../assets/images/eye-off.svg';
import Eye from '../../assets/images/eye.svg';
import { TextInput } from 'react-native-paper';
import { View, StyleSheet, Text } from 'react-native';
import colors from '../../styles/colors';
import { FontAwesome } from '@expo/vector-icons';
import Popover, { PopoverPlacement } from 'react-native-popover-view';


interface CustomInputProps {
  readonly state?: string;
  readonly label: string;
  readonly handleChange?: (e: string) => void;
  readonly leftIcon?: any;
  readonly rightIcon?: any;
  readonly withTooltip?: boolean;
  readonly readonly?: boolean;
  readonly isInvalid?: boolean;
  readonly size?: number;
  readonly tooltipText?: string;
  readonly type?: string;
  readonly maxLength?: number;
	readonly id?: string;
}

const getRightIcon = (
  type: string,
  isHidden: any,
  setIsHidden: (arg0: boolean) => void,
  rightIcon:
    | string
    | number
    | boolean
    | React.ReactElement<any, string | React.JSXElementConstructor<any>>
    | Iterable<React.ReactNode>
    | React.ReactPortal
    | null
    | undefined
) => {
  if (type === 'password') {
    return (
      <TextInput.Icon
        onPress={() => setIsHidden(!isHidden)}
        icon={() => (isHidden ? <EyeOff /> : <Eye />)}
      />
    );
  } else if (rightIcon) {
    return <TextInput.Icon icon={() => rightIcon} />;
  }
  return null;
};

const getTooltip = (
  tooltipRef: any,
  setTooltipOpen:  React.Dispatch<React.SetStateAction<boolean>>
) => {
  return (
    <TextInput.Icon
      style={{padding: 5}}
      onPress={() => setTooltipOpen(true)}
      icon={() => (
        <FontAwesome ref={tooltipRef} name="info-circle" size={18} style={styles.icon} />
      )}
    />
  );

};

export default function CustomInput({
  state = '',
  handleChange,
  leftIcon,
  rightIcon,
  label,
  readonly = false,
  isInvalid = false,
  withTooltip = false,
  size = 1,
  tooltipText = 'dica padrao',
  type = 'text',
  maxLength,
	id = '',
}: CustomInputProps) {
  const [isHidden, setIsHidden] = useState(type === 'password');


  const isMultiline = size === 2;

  const rightIconComponent = getRightIcon(type, isHidden, setIsHidden, rightIcon);
  const [tooltipOpen, setTooltipOpen] = useState(false);
  const tooltipRef = useRef(null);
  const tooltipComponent = getTooltip(tooltipRef, setTooltipOpen);

  return (
    <View style={styles.container}>
      <View style={styles.inputContainer}>
        <TextInput
          id={id}
          mode="outlined"
          outlineColor="transparent"
          textAlignVertical={isMultiline ? 'top' : 'center'}
          style={{ ...styles.textContainer }}
          numberOfLines={isMultiline ? 4 : 1}
          placeholderTextColor={colors.EERIE_BLACK}
          label={<Text style={{ color: colors.EERIE_BLACK }}>{label}</Text>}
          value={state}
          outlineStyle={{ borderColor: 'rgba(32, 26, 23, 0.12)', borderWidth: 1, borderRadius: 4 }}
          secureTextEntry={isHidden}
          onChangeText={handleChange}
          error={isInvalid}
          multiline={isMultiline}
          disabled={readonly}
          maxLength={maxLength}
          contentStyle={{
            color: colors.EERIE_BLACK,
          }}
          left={
            leftIcon ? (
              <TextInput.Icon
                style={{
                  pointerEvents: 'none',
                  display: 'flex',
                  justifyContent: 'center',
                }}
                icon={() => leftIcon}
              />
            ) : null
          }
          right={withTooltip ? tooltipComponent : rightIconComponent}
        />
      </View>
      <Popover placement={PopoverPlacement.BOTTOM} onRequestClose={() => { setTooltipOpen(false) }} isVisible={tooltipOpen} from={tooltipRef}>
        <Text style={{ textAlign: "left", color: "black", padding: 5 }}>{tooltipText}</Text>
      </Popover>
    </View>
  );
}

const styles = StyleSheet.create({
  icon: {
    color: colors.SELECTIVE_YELLOW,
    zIndex: 10
  },
  container: {
    display: 'flex',
    width: '100%',
    flexDirection: 'row',
    alignItems: 'center',
    shadowColor: 'rgba(0, 0, 0, 0.25)',
    shadowOffset: {
      width: 0,
      height: 3,
    },
    shadowOpacity: 0.3,
    shadowRadius: 2,
    elevation: 5,
  },
  inputContainer: {
    flex: 1,
  },
  textContainer: {
    backgroundColor: colors.WHITE,
    opacity: 1,
    textDecorationLine: 'none',
    display: 'flex',
    borderRadius: 10,
  },
  tooltipContainer: {
    position: 'absolute',
    right: -5,
    top: '55%',
    transform: [{ translateY: -10 }],
    flexDirection: 'row',
    alignItems: 'center',
    display: 'flex',
  },
  tooltipText: {
    marginLeft: 5,
  },
});
